Package: pec
Title: Prediction Error Curves for Risk Prediction Models in Survival
        Analysis
Version: 2025.06.24
Authors@R: person(given = c("Thomas", "A."),
                    family = "Gerds",
                    role = c("aut", "cre"),
                    email = "tag@biostat.ku.dk")
Description: Validation of risk predictions obtained from survival models and
    competing risk models based on censored data using inverse weighting and
    cross-validation. Most of the 'pec' functionality has been moved to 'riskRegression'.
Depends: R (>= 2.9.0), prodlim (>= 1.4.9)
Imports: foreach (>= 1.4.2), rms (>= 4.2-0), survival (>= 2.37-7),
        riskRegression (>= 2020.02.05), lava (>= 1.4.1), timereg (>=
        1.8.9),
Suggests: party, cmprsk (>= 2.2-7), rpart, Hmisc (>= 3.14-4)
Enhances: randomForestSRC
Maintainer: Thomas A. Gerds <tag@biostat.ku.dk>
License: GPL (>= 2)
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2025-07-24 07:06:19 UTC; tag
Author: Thomas A. Gerds [aut, cre]
Repository: CRAN
Date/Publication: 2025-07-24 14:30:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-30 18:47:00 UTC; unix
Archs: pec.so.dSYM
