# Generated by roxygen2: do not edit by hand

S3method(ipcw,aalen)
S3method(ipcw,cox)
S3method(ipcw,forest)
S3method(ipcw,marginal)
S3method(ipcw,none)
S3method(ipcw,nonpar)
S3method(ipcw,rfsrc)
S3method(plot,Cindex)
S3method(plot,calibrationPlot)
S3method(plot,confScoreSurv)
S3method(plot,pec)
S3method(plot,riskReclassification)
S3method(predictEventProb,ARR)
S3method(predictEventProb,CauseSpecificCox)
S3method(predictEventProb,FGR)
S3method(predictEventProb,coxboost)
S3method(predictEventProb,matrix)
S3method(predictEventProb,prodlim)
S3method(predictEventProb,pseudoForest)
S3method(predictEventProb,rfsrc)
S3method(predictEventProb,riskRegression)
S3method(predictLifeYearsLost,ARR)
S3method(predictLifeYearsLost,CauseSpecificCox)
S3method(predictLifeYearsLost,FGR)
S3method(predictLifeYearsLost,coxboost)
S3method(predictLifeYearsLost,matrix)
S3method(predictLifeYearsLost,prodlim)
S3method(predictLifeYearsLost,rfsrc)
S3method(predictLifeYearsLost,riskRegression)
S3method(predictProb,glm)
S3method(predictProb,ols)
S3method(predictProb,randomForest)
S3method(predictRestrictedMeanTime,aalen)
S3method(predictRestrictedMeanTime,cox.aalen)
S3method(predictRestrictedMeanTime,coxph)
S3method(predictRestrictedMeanTime,coxph.penal)
S3method(predictRestrictedMeanTime,cph)
S3method(predictRestrictedMeanTime,default)
S3method(predictRestrictedMeanTime,matrix)
S3method(predictRestrictedMeanTime,numeric)
S3method(predictRestrictedMeanTime,pecRpart)
S3method(predictRestrictedMeanTime,prodlim)
S3method(predictRestrictedMeanTime,psm)
S3method(predictRestrictedMeanTime,rfsrc)
S3method(predictRestrictedMeanTime,riskRegression)
S3method(predictRestrictedMeanTime,selectCox)
S3method(predictRestrictedMeanTime,survfit)
S3method(predictSurvProb,aalen)
S3method(predictSurvProb,cox.aalen)
S3method(predictSurvProb,coxboost)
S3method(predictSurvProb,coxph)
S3method(predictSurvProb,cph)
S3method(predictSurvProb,default)
S3method(predictSurvProb,matrix)
S3method(predictSurvProb,numeric)
S3method(predictSurvProb,pecCforest)
S3method(predictSurvProb,pecCtree)
S3method(predictSurvProb,pecRpart)
S3method(predictSurvProb,penfitS3)
S3method(predictSurvProb,prodlim)
S3method(predictSurvProb,pseudoForest)
S3method(predictSurvProb,psm)
S3method(predictSurvProb,rfsrc)
S3method(predictSurvProb,riskRegression)
S3method(predictSurvProb,selectCox)
S3method(predictSurvProb,survfit)
S3method(print,Cindex)
S3method(print,IPCW)
S3method(print,R2)
S3method(print,calibrationPlot)
S3method(print,confScoreSurv)
S3method(print,crps)
S3method(print,method)
S3method(print,multiSplitTest)
S3method(print,pec)
S3method(print,riskReclassification)
S3method(print,splitMethod)
S3method(print,vandeWielTest)
S3method(summary,Cindex)
S3method(summary,confScoreSurv)
S3method(summary,pec)
export(R2)
export(calPlot)
export(cindex)
export(coxboost)
export(crps)
export(ibs)
export(ipcw)
export(pec)
export(pecCforest)
export(pecCtree)
export(pecRpart)
export(plotPredictEventProb)
export(plotPredictSurvProb)
export(predictEventProb)
export(predictLifeYearsLost)
export(predictRestrictedMeanTime)
export(predictSurvProb)
export(resolvesplitMethod)
export(selectCox)
export(simCost)
importFrom(foreach,"%dopar%")
importFrom(grDevices,col2rgb)
importFrom(grDevices,gray)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,segments)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(prodlim,Hist)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,family)
importFrom(stats,formula)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.fail)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rexp)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,smooth)
importFrom(stats,terms)
importFrom(stats,time)
importFrom(stats,update)
importFrom(stats,update.formula)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(survival,Surv)
importFrom(timereg,aalen)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,select.list)
useDynLib(pec, .registration=TRUE)
