% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.pec.R
\name{print.pec}
\alias{print.pec}
\alias{summary.pec}
\title{Printing a `pec' (prediction error curve) object.}
\usage{
\method{print}{pec}(x, times, digits = 3, what = NULL, ...)
}
\arguments{
\item{x}{Object of class \code{pec}}

\item{times}{Time points at which to show the values of the prediction error
curve(s)}

\item{digits}{Number of decimals used in tables.}

\item{what}{What estimate of the prediction error curve to show. Should be a
string matching an element of x. The default is determined by
splitMethod.}

\item{\dots}{Not used}

\item{print}{Set to FALSE to suppress printing.}
}
\value{
The first argument in the invisible cloak.
}
\description{
Print the important arguments of call and the prediction error values at
selected time points.
}
\seealso{
\code{\link{pec}}
}
\author{
Thomas A. Gerds \email{tag@biostat.ku.dk}
}
\keyword{survival}
